import os
import pytest
import shutil
from abc import ABC, abstractmethod

from idmtools_test.utils.itest_with_persistence import ITestWithPersistence

from emodpy.utils import download_latest_bamboo, download_latest_schema, EradicationBambooBuilds, bamboo_api_login, \
    download_from_url
import emodpy.bamboo_api_utils as bamboo_api

import sys
file_dir = os.path.dirname(__file__)
sys.path.append(file_dir)
import manifest


#@pytest.mark.skip('remove this line when running locally with pytest')
# skip this one for github action for now, download_from_bamboo need to support these the following features before
# we can run these E2E tests in github action:
# login to bamboo in python script
class TestBambooDownload(ITestWithPersistence, ABC):
    @abstractmethod
    def define_test_environment(self):
        self.plan = EradicationBambooBuilds.CI_GENERIC
        self.eradication_path = manifest.eradication_path_win
        self.schema_path = manifest.schema_path_win

    def setUp(self) -> None:
        self.define_test_environment()
        self.case_name = os.path.basename(__file__) + "--" + self._testMethodName
        print(self.case_name)
        print('login to bamboo...')
        bamboo_api_login()

    def get_eradication_test(self):
        manifest.delete_existing_file(self.eradication_path)
        download_latest_bamboo(
            plan=self.plan, scheduled_builds_only=False, out_path=self.eradication_path
        )
        self.assertTrue(os.path.isfile(self.eradication_path), msg=f'{self.eradication_path} is not exist.')

    def get_schema_test(self):
        manifest.delete_existing_file(self.schema_path)
        download_latest_schema(
            plan=self.plan, scheduled_builds_only=False, out_path=self.schema_path
        )
        self.assertTrue(os.path.isfile(self.schema_path), msg=f'{self.schema_path} is not exist.')


@pytest.mark.bamboo
class TestBambooDownloadGenWin(TestBambooDownload):
    """
    Tested with Windows version of Generic Eradication
    """
    def define_test_environment(self):
        self.plan = EradicationBambooBuilds.CI_GENERIC
        self.eradication_path = manifest.eradication_path_win
        self.schema_path = manifest.schema_path_win

    def test_bamboo_download_eradication_gen_win(self):
        super().get_eradication_test()

    def test_bamboo_download_schema_gen_win(self):
        super().get_schema_test()


@pytest.mark.bamboo
class TestBambooDownloadGenLinux(TestBambooDownload):
    """
    Tested with Linux version of Generic Eradication
    """
    def define_test_environment(self):
        self.plan = EradicationBambooBuilds.CI_GENERICLINUX
        self.eradication_path = manifest.eradication_path_linux
        self.schema_path = manifest.schema_path_linux

    def test_bamboo_download_eradication_gen_linux(self):
        super().get_eradication_test()

    def test_bamboo_download_schema_gen_linux(self):
        super().get_schema_test()



@pytest.mark.bamboo
class TestBambooDownloadGenWinALL(TestBambooDownload):
    """
    Tested with Windows version of All Diseases Eradication
    """
    def define_test_environment(self):
        self.plan = EradicationBambooBuilds.GENERIC_WIN_ALL
        self.eradication_path = manifest.eradication_path_win_all
        self.schema_path = manifest.schema_path_win_all

    def test_bamboo_download_eradication_gen_win_all(self):
        super().get_eradication_test()

    def test_bamboo_download_schema_gen_win_all(self):
        super().get_schema_test()


@pytest.mark.bamboo
class TestBambooDownloadURL(ITestWithPersistence):
    """
    Tested with url
    """

    @classmethod
    def setUpClass(cls) -> None:
        cls.plan = EradicationBambooBuilds.CI_GENERICLINUX
        bamboo_api_login()
        build_num, build_info = bamboo_api.BuildInfo.get_latest_successful_build(cls.plan.value,
                                                                                 scheduled_only=False)
        if not build_num:
            raise FileNotFoundError(
                f"Could not find a successful build for plan {cls.plan.value}. Please check plan name again")
        base_bamboo_url = f"http://idm-bamboo:8085/artifact/{cls.plan.value}/shared/build-{str(build_num).zfill(5)}/"
        # eradication bamboo path
        cls.eradication_url = base_bamboo_url + "Eradication.exe/build/x64/Release/Eradication/Eradication"

        # schema bamboo path
        cls.schema_url = base_bamboo_url + 'schema.json/DtkTrunk/schema.json.txt'

        # dll bamboo path
        cls.dll_base_url = base_bamboo_url + "Reporter-Plugins/build/x64/Release/reporter_plugins/"
        # dll list files
        cls.dll_list = ["libReportLineList.so", "libreporteventcounter.so "]

    def setUp(self) -> None:
        self.case_name = os.path.basename(__file__) + "--" + self._testMethodName
        print(self.case_name)

    def test_bamboo_download_eradication_url(self):
        manifest.delete_existing_file(os.path.join(manifest.bin_folder, 'Eradication'))
        download_from_url(self.eradication_url, manifest.bin_folder)
        self.assertTrue(os.path.isfile(os.path.join(manifest.bin_folder, 'Eradication')))
        shutil.move(os.path.join(manifest.bin_folder, 'Eradication'),
                    manifest.eradication_path_linux_latest)

    def test_bamboo_download_schema_url(self):
        manifest.delete_existing_file(os.path.join(manifest.schema_folder, 'schema.json.txt'))
        download_from_url(self.schema_url, manifest.schema_folder)
        self.assertTrue(os.path.isfile(os.path.join(manifest.schema_folder, 'schema.json.txt')))
        shutil.move(os.path.join(manifest.schema_folder, 'schema.json.txt'),
                    manifest.schema_path_linux_latest)

    def test_bamboo_download_dll_url(self):
        for dll_filename in self.dll_list:
            manifest.delete_existing_file(os.path.join(manifest.plugins_folder, dll_filename))
            download_from_url(self.dll_base_url + dll_filename, manifest.plugins_folder)
            self.assertTrue(os.path.isfile(os.path.join(manifest.plugins_folder, dll_filename)))


if __name__ == "__main__":
    import unittest
    unittest.main()

