# flake8: noqa W605,F821
import copy
import json
import os
import pytest
import pdb
from idmtools.entities.experiment import Experiment
from idmtools.entities.simulation import Simulation
from idmtools_test import COMMON_INPUT_PATH
from idmtools_test.utils.itest_with_persistence import ITestWithPersistence
from emodpy.emod_task import EMODTask
from emod_api.config import from_schema as fs

import sys
file_dir = os.path.dirname(__file__)
sys.path.append(file_dir)
import manifest

# current_directory = os.path.dirname(os.path.realpath(__file__))
# DEFAULT_CONFIG_PATH = os.path.join(COMMON_INPUT_PATH, "files", "config.json")
# DEFAULT_CAMPAIGN_JSON = os.path.join(COMMON_INPUT_PATH, "files", "campaign.json")
# DEFAULT_DEMOGRAPHICS_JSON = os.path.join(COMMON_INPUT_PATH, "files", "demographics.json")
# DEFAULT_ERADICATION_PATH = os.path.join(COMMON_INPUT_PATH, "emod", "Eradication.exe")


@pytest.mark.emod
class TestEMODTask(ITestWithPersistence):

    def setUp(self) -> None:
        self.case_name = os.path.basename(__file__) + "--" + self._testMethodName
        print(self.case_name)

    def test_command(self):
        models_dir_list = [manifest.eradication_path_linux_latest,
                           manifest.eradication_path_win_all,
                           manifest.eradication_path_win,
                           manifest.eradication_path_linux]
        for models_dir in models_dir_list:
            print(f'testing {models_dir} ...')
            task = EMODTask(eradication_path=models_dir)
            task.pre_creation(Simulation())
            eradication_name = os.path.basename(models_dir)
            self.assertEqual(
                f'Assets/{eradication_name} --python-script-path ./Assets/python --config config.json --input-path "./Assets;." --dll-path ./Assets',
                task.command.cmd)

    """
    def test_legacy_emod(self):
        models_dir = os.path.join(COMMON_INPUT_PATH, "fakemodels", "Eradication.exe")
        task = EMODTask(eradication_path=models_dir)
        task.pre_creation(Simulation)
        #self.assertEqual(r"Assets/Eradication.exe --config config.json --input-path ./Assets;. --dll-path ./Assets", task.command.cmd)

        models_dir = os.path.join(COMMON_INPUT_PATH, "fakemodels", "Eradication.exe")
        task = EMODTask(eradication_path=models_dir, legacy_exe=True)
        task.pre_creation(Simulation)
        print( "task.command.cmd = " + task.command.cmd )
        self.assertEqual(r"Assets/Eradication.exe --config config.json --input-path ./Assets --dll-path ./Assets", task.command.cmd)
    """

    @staticmethod
    def generate_default_config(config_name, schema_path=manifest.schema_path_linux):
        # generate config file from emod-api
        config_path = os.path.join(manifest.config_folder, config_name)
        manifest.delete_existing_file(config_path)
        builder = fs.SchemaConfigBuilder(schema_name=schema_path, config_out=config_path)
        return config_path

    @staticmethod
    def generate_campaign(schema_path, camp_path="campaign.json"):
        # generate campaign file from emod-api
        from emod_api.interventions import outbreak as ob
        from emod_api import campaign as camp

        ob.schema_path = schema_path
        event = ob.new_intervention(1, cases=4)
        camp.add(event)
        # We are saving and reloading. Maybe there's an even better way? But even an outbreak seeding does not belong in the EMODTask.
        camp.save(camp_path)

    def test_load_files(self):
        schema_path = manifest.schema_path_linux
        print(f"Telling emod-api to use {schema_path} as schema.")

        config_path = self.generate_default_config(config_name='config_emod_experiment_test_load_files.json',
                                                   schema_path=schema_path)
        camp_path = "campaign_load_files.json"
        self.generate_campaign(schema_path=schema_path, camp_path=camp_path)

        eradication_path = manifest.eradication_path_linux
        task = EMODTask.from_files(
            eradication_path=eradication_path,
            config_path=config_path,
            campaign_path=camp_path,
            demographics_paths=None  # ToDo: generate a demo file from emodapi to test. DEFAULT_DEMOGRAPHICS_JSON
        )

        task.pre_creation(Simulation())
        task.gather_common_assets()

        experiment = Experiment.from_task(copy.deepcopy(task), name=self.case_name)

        # Open all the files for comparison
        with open(config_path, 'r') as fp:
            config = json.load(fp)["parameters"]
        with open(camp_path, 'r') as fp:
            campaign = json.load(fp)

        # Todo: load the demo file
        # with open(DEFAULT_DEMOGRAPHICS_JSON, 'r') as fp:
        #     demog = json.load(fp)

        task.gather_common_assets()

        self.assertEqual(task.eradication_path, eradication_path)
        self.assertIn(eradication_path, [a.absolute_path for a in task.common_assets.assets])
        # Todo: assert the demo file
        # self.assertIn(DEFAULT_DEMOGRAPHICS_JSON, [a.absolute_path for a in task.common_assets.assets])
        # self.assertDictEqual(demog,
        #                      task.common_assets.get_one(filename=os.path.basename(DEFAULT_DEMOGRAPHICS_JSON)).content)

        # check experiment common assets are as expected
        experiment.pre_creation()
        self.assertIn(eradication_path, [a.absolute_path for a in experiment.assets])
        # Todo: assert the demo file
        # self.assertIn(DEFAULT_DEMOGRAPHICS_JSON, [a.absolute_path for a in experiment.assets])

        sim = experiment.simulations[0]
        # Todo: assert the demo file after pre_creation.
        # # After pre_creation, the demographics filenames will be set on the sim so no more equality
        sim.pre_creation()
        # self.assertNotEqual(config["Demographics_Filenames"], sim.task.config["Demographics_Filenames"])
        # # Are the demographics set correctly?
        # self.assertEqual(sim.task.config["Demographics_Filenames"],
        #                  [os.path.join("demographics", os.path.basename(DEFAULT_DEMOGRAPHICS_JSON))])

        # The rest should be the same
        # Todo: delete demo file name when it's ready
        # del config["Demographics_Filenames"]
        # del sim.task.config["Demographics_Filenames"]
        self.assertEqual(sim.task.config["parameters"]["Enable_Vital_Dynamics"], 1)
        del sim.task.config["parameters"]["Enable_Vital_Dynamics"]
        #self.set_migrations(config)
        #self.assertDictEqual(config, sim.task.config)

        # Todo: assert the demo file
        # # The demographics coming from the experiment
        # self.assertIsNone(sim.task.transient_assets.get_one(filename=os.path.basename(DEFAULT_DEMOGRAPHICS_JSON)))
        # self.assertIsNotNone(sim.task.demographics.get_one(filename=os.path.basename(DEFAULT_DEMOGRAPHICS_JSON)))

        # No change for campaigns
        #pdb.set_trace().
        self.assertEqual(campaign['Events'], task.campaign.events)
        #self.assertEqual(campaign['Campaign_Name'], task.campaign.name)

    def test_load_files_without_demographics(self):
        schema_path = manifest.schema_path_linux
        print(f"Telling emod-api to use {schema_path} as schema.")

        config_path = self.generate_default_config(config_name='config_emod_experiment_test_load_files.json',
                                                   schema_path=schema_path)
        camp_path = "campaign_load_files_without_demographics.json"
        self.generate_campaign(schema_path=schema_path, camp_path=camp_path)

        eradication_path = manifest.eradication_path_linux
        task = EMODTask.from_files(eradication_path=eradication_path,
                                   config_path=config_path,
                                   campaign_path=camp_path)

        with open(config_path, 'r') as fp:
            config = json.load(fp)["parameters"]
        with open(camp_path, 'r') as fp:
            campaign = json.load(fp)

        self.assertDictEqual(config, task.config)
        self.assertEqual(campaign['Events'], task.campaign.events)

    def set_migrations(self, dict):
        dict.update({'Enable_Local_Migration': 0})
        dict.update({'Enable_Air_Migration': 0})
        dict.update({'Enable_Family_Migration': 0})
        dict.update({'Enable_Regional_Migration': 0})
        dict.update({'Enable_Sea_Migration': 0})
        return dict
