import json
import os

import pytest

from idmtools.core import ItemType
from idmtools.core.platform_factory import Platform
from idmtools.entities.experiment import Experiment
from idmtools_test.utils.itest_with_persistence import ITestWithPersistence

from emodpy.emod_task import EMODTask
from emodpy.reporters.custom import Report_TBHIV_ByAge
from examples.config_update_parameters import del_folder
from examples.serialization.globals import config_update_params

current_directory = os.path.dirname(os.path.realpath(__file__))
INPUT_PATH = os.path.join(current_directory, "..", "examples", "reporters", "inputs")
BIN_PATH = os.path.join(current_directory, "..", "examples", "inputs", "bin")


@pytest.mark.comps
class TestReports(ITestWithPersistence):

    def setUp(self) -> None:
        self.case_name = os.path.basename(__file__) + "--" + self._testMethodName
        print(self.case_name)
        self.platform = Platform('COMPS2')

    def test_custom_report(self):
        # Create EMODTask with the set of provided files
        task = EMODTask.from_files(eradication_path=os.path.join(INPUT_PATH, "Eradication.exe"),
                                   config_path=os.path.join(INPUT_PATH, "config.json"),
                                   campaign_path=os.path.join(INPUT_PATH, "campaign.json"),
                                   demographics_paths=[os.path.join(INPUT_PATH, "Base_Demog_Trunk_TB.json"),
                                                       os.path.join(INPUT_PATH, "Base_Overlay_TB.json")])

        # This is a legacy executable -> set it in the task
        # Points the reporters to the correct dll path
        task.reporters.add_dll_folder(os.path.join(INPUT_PATH))

        # Create a report TBHIV by Age and add a couple of reports to it
        report = Report_TBHIV_ByAge()
        report.add_report(200, 0, 0, 200)
        report.add_report(100, 0, 0, 100)

        task.reporters.add_reporter(report)

        # Create the experiment from task and run
        experiment = Experiment.from_task(task, name=self.case_name)
        self.platform.run_items(experiment)
        self.platform.wait_till_done(experiment)
        self.assertTrue(experiment.succeeded, "expected experiment succeeded")

        # verify:
        # 1) download output/Report_TBHIV_ByAge.csv from simulation, verify download file exists
        # 2) download 'custom_reports.json' from simulation working dir, validate it has correct content

        # delete output from previous run
        del_folder("output")
        # create a new empty 'output' dir
        os.mkdir("output")
        filenames = ["output/Report_TBHIV_ByAge.csv", "custom_reports.json"]

        sims = self.platform.get_children_by_object(experiment)
        for simulation in sims:
            # download files from simulation
            self.platform.get_files_by_id(simulation.id, item_type=ItemType.SIMULATION, files=filenames,
                                          output='output')
            # validate files exist
            local_path = os.path.join("output", str(simulation.uid))
            self.assertTrue(os.path.exists(os.path.join(local_path, "output", "Report_TBHIV_ByAge.csv")))
            custom_reports_file_path = os.path.join(local_path, "custom_reports.json")
            self.assertTrue(os.path.exists(custom_reports_file_path))

            with open(custom_reports_file_path, "r") as f:
                custom_reports = json.load(f)
                print(custom_reports)
                reports = custom_reports["Custom_Reports"]["Report_TBHIV_ByAge"]["Reports"]
                self.assertDictEqual(reports[0],
                                     {'Max_Age_Yrs': 200, 'Min_Age_Yrs': 0, 'Start_Year': 0, 'Stop_Year': 200})
                self.assertDictEqual(reports[1],
                                     {'Max_Age_Yrs': 100, 'Min_Age_Yrs': 0, 'Start_Year': 0, 'Stop_Year': 100})

    def test_multiple_custom_reports(self):
        input = os.path.abspath(os.path.join(current_directory, "..", "examples", "serialization"))
        task = EMODTask.from_files(eradication_path=os.path.join(BIN_PATH, "Eradication.exe"),
                                   config_path=os.path.join(input, "inputs", "config.json"),
                                   campaign_path=os.path.join(input, "inputs", "campaign.json"),
                                   demographics_paths=os.path.join(input, "inputs", "3nodes_demographics.json"))

        config_update_params(task)

        # Add the DLLs to the collection
        task.reporters.add_dll_folder(os.path.join(input, "custom_reports", "reporter_plugins", "Windows"))
        task.reporters.read_custom_reports_file(os.path.join(input, "custom_reports", "custom_reports.json"))
        self.assertTrue(task.reporters.custom_reporters.count, 2)
        self.assertTrue(task.reporters.custom_reporters[0].name, 'ReportHumanMigrationTracking')
        self.assertTrue(task.reporters.custom_reporters[1].name, 'ReportNodeDemographics')

        experiment = Experiment.from_task(task, name=self.case_name)
        self.platform.run_items(experiment)
        self.platform.wait_till_done(experiment)
        self.assertTrue(experiment.succeeded, "expected experiment succeeded")

        # verify 2 custom reports in output of simulation
        # 1) download output/ReportHumanMigrationTracking.csv from simulation, verify file exists
        # 2) download output/ReportNodeDemographics.csv from simulation, validate file exists
        # 3) download output/DemographicsSummary.json from simulation, verify file exists

        # delete output from previous run
        del_folder("output")
        # create a new empty 'output' dir
        os.mkdir("output")
        filenames = ["output/ReportHumanMigrationTracking.csv", "output/ReportNodeDemographics.csv",
                     "output/DemographicsSummary.json"]

        sims = self.platform.get_children_by_object(experiment)
        for simulation in sims:
            # download files from simulation
            self.platform.get_files_by_id(simulation.id, item_type=ItemType.SIMULATION, files=filenames,
                                          output='output')
            # validate files exist
            local_path = os.path.join("output", str(simulation.uid), "output")
            self.assertTrue(os.path.exists(os.path.join(local_path, "ReportHumanMigrationTracking.csv")))
            self.assertTrue(os.path.exists(os.path.join(local_path, "ReportNodeDemographics.csv")))
            self.assertTrue(os.path.exists(os.path.join(local_path, "DemographicsSummary.json")))
